"""
We have a ``MinipubAdmin`` class that basically gives you the code for displaying 2
fieldsets:

#. The fieldset that controls publication:

.. image:: /img/minipub-fieldset.png

2. And the 'status' fieldset; these fields are read-only, and exist just for information:

.. image:: /img/minipub-status-fieldset.png

And here is an example of using them:

.. code-block:: python

    from minipub.admin import MinipubAdmin

    class ArticleAdmin(MinipubAdmin):

    # Other code...

    fieldsets = (
        (None, {
            'fields': ('some', 'other', 'fields',)
        }),
        MinipubAdmin.PUBLICATION_FIELDSET,
        MinipubAdmin.TIMESTAMP_FIELDSET
    )
"""
from django.contrib import admin


class MinipubAdmin(admin.ModelAdmin):

    readonly_fields = ('created', 'modified', 'status_changed')
    ordering = ['-start']

    TIMESTAMP_FIELDSET = ('Timestamps', {
        'description': 'When this record was created, last modified, and when '
        'the status was last changed.',
        'classes': ('collapse',),
        'fields': (('created', 'modified', 'status_changed'),)
    })

    PUBLICATION_FIELDSET = ('Publication', {
        'description': "You can control here the publication of this object.<br>"
        "An object is only 'live' if its status is 'published', <strong>and</strong> "
        "if today's date is after the start date. "
        "If the end date is entered, the object will no longer be live after that date.<br>"
        "<strong>Note:</strong> when you are logged in to the 'admin', you can preview "
        "this object in the website, even if it's not published.",
        'fields': (('status', 'start', 'end'),)
    })



"""

.. code-block:: python

    from minipub.models import MinipubModel

    class Article(MinipubModel):
        ...

``MinipubModel`` is an abstract model that provides the following 3 fields:

- ``status``: a list of choices; default is 'draft' or 'published'.
- ``start``: a start date; defaults to date of publication.
- ``end``: an end date; optional.

Timestamps
----------
``MinipubModel`` also adds the following fields that get auto-updated and should
not be manually modified: ``created``, ``modified`` and ``status_changed``.

The concept of 'Live' objects
-----------------------------

Objects are usually considered 'live' **if** they are 'published'
**and** between the start and end dates - this is usually sufficient for them being available
to display in the public website.

``live()`` methods are available both as a chainable filter on a queryset,
and as an instance method. For example, if you have an ``Article`` model that uses ``MinipubModel``:

.. code-block:: python

    my_articles = Article.objects.live()

or

.. code-block:: python

    can_be_viewed = article1.live()

Extra statuses
~~~~~~~~~~~~~~
Models can have more statuses than ``draft``, ``published`` -
:ref:`see here for more details<extra_statuses-label>`.

Sitemaps
~~~~~~~~
If you have defined a sitemap.xml, refer also to the :ref:`sitemaps page<sitemaps-label>`.

"""


from django.db import models
from django.core.exceptions import ValidationError
from django import VERSION

from model_utils.models import StatusModel, TimeStampedModel
from model_utils import Choices

import datetime

from .managers import MinipubQuerySet


class MinipubModel(StatusModel, TimeStampedModel):

    STATUS = Choices('draft', 'published')

    start = models.DateField('start date', null=True, blank=True)
    end = models.DateField('end date', null=True, blank=True)

    objects = MinipubQuerySet.as_manager()

    class Meta:
        abstract = True
        if VERSION[0] == 1 and VERSION[1] >= 10:
            default_manager_name = 'objects'

    def save(self, *args, **kwargs):
        """Set the start date for non-draft items if it hasn't been set already."""
        if self.status != self.STATUS.draft and self.start is None:
            self.start = datetime.date.today()
        super(MinipubModel, self).save(*args, **kwargs)

    def clean(self):
        super(MinipubModel, self).clean()
        if self.start and self.end and self.start > self.end:
            raise ValidationError('The end date cannot be before the start date.')

    def live(self, statuses=['published']):
        if self.status not in statuses:
            return False
        if self.start and self.start > datetime.date.today():
            return False
        if self.end and self.end < datetime.date.today():
            return False
        return True
    live.boolean = True

    @property
    def staff_preview(self):
        """Helper property - says if this object is being previewed by a member of staff.

        Can be used when displaying objects in the website - members of staff will see all
        objects, using this property in the template can help for attaching a message/custom
        CSS saying that this object is being previewed.

        For example, in the example_project we have this snippet:

        .. code-block:: django

            {% if article.staff_preview %}
                <div class="label label-warning">This is a preview</div>
            {% endif %}

        """

        return self.status == self.STATUS.draft



from django.db.models.query import QuerySet
from django.db.models import Q

import datetime


class MinipubQuerySet(QuerySet):

    def live(self, statuses=['published']):
        today = datetime.date.today()
        return self.filter(status__in=statuses).\
            filter(Q(start__lte=today) | Q(start__isnull=True)).\
            filter(Q(end__gte=today) | Q(end__isnull=True))



"""

.. code-block:: python

    from minipub.views import MinipubArchiveIndexView, MinipubYearArchiveView, MinipubDetailView

    class ArticleArchiveView(MinipubArchiveIndexView):
        ...

Minipub provides a few basic views to get you started - but you are encouraged to look at the source
code. The important thing to note is the ``GetQuerysetMixin`` mixin - you can use this with most
of Django's List- and Detail- class based-views to easily integrate minipub.

For example, in minipub' ``views.py``, here is the source code for defining a detail view:

.. code-block:: python

    class MinipubDetailView(GetQuerysetMixin, DetailView):
        pass

"""

from django.views.generic.dates import ArchiveIndexView, YearArchiveView
from django.views.generic import DetailView


class GetQuerysetMixin(object):

    minipub_live = ('published',)

    def get_queryset(self):
        qs = super(GetQuerysetMixin, self).get_queryset()
        # Staff users are a special case - we want them to be able to see an article
        # (most) of the time, so that they can review it before it goes live.
        # The exception is when an article can have extra statuses - e.g. draft, published,
        # archived. In this case, we only want the show the status that is applicable to
        # this page, *plus* the draft status.
        if self.request.user.is_authenticated() and self.request.user.is_staff:
            staff_statuses = self.minipub_live + (self.model.STATUS.draft,)
            return qs.filter(status__in=staff_statuses)
        return qs.live(statuses=self.minipub_live)


class MinipubArchiveIndexView(GetQuerysetMixin, ArchiveIndexView):
    date_field = 'start'


class MinipubYearArchiveView(GetQuerysetMixin, YearArchiveView):
    date_field = 'start'


class MinipubDetailView(GetQuerysetMixin, DetailView):
    pass



'''
.. _sitemaps-label:

We have a ``MinipubSitemap`` class that provides a small amount of boilerplate
to make creating a sitemap easier.

Here is an example of how to use it:

.. code-block:: python

    from minipub.sitemaps import MinipubSitemap

    from .models import Article

    class NewsSitemap(MinipubSitemap):
        model = Article

Basically, all you'll need to do is set the model to use in the sitemap - in
this example, it's ``Article``.

If you have custom statuses...
------------------------------
If you have more statuses than the default 'draft', 'published', and you have different
sets of views to display these - you will also need extra sitemap classes.

You can add a ``minipub_live`` attribute to your sitemap class - exactly
the same way as you will have done for your extra views. For example:

.. code-block:: python

    from minipub.sitemaps import MinipubSitemap

    from .models import Article

    class NewsArchivesSitemap(MinipubSitemap):
        """Sitemap for the archived articles."""
        model = Article
        minipub_live = ('archived',)

'''
from django.contrib.sitemaps import Sitemap


class MinipubSitemap(Sitemap):

    minipub_live = ('published',)

    def items(self):
        return self.model.objects.live(statuses=self.minipub_live)

    def lastmod(self, obj):
        return obj.modified



#/usr/bin/env python

import uuid
from setuptools import setup, find_packages
from pip.req import parse_requirements

import minipub


def get_requirements(source):
    try:
        install_reqs = parse_requirements(source, session=uuid.uuid1())
    except TypeError:
        # Older version of pip.
        install_reqs = parse_requirements(source)
    return set([str(ir.req) for ir in install_reqs])

setup(
    name='django-minipub',
    version=minipub.__version__,
    packages=find_packages(exclude=['example_project']),
    include_package_data=True,
    license='MIT',
    description='Django-minipub is a MINImalist PUBlication control system for Django.',
    long_description=open('README.rst').read(),
    install_requires=get_requirements('requirements.txt'),
    url='https://github.com/richardbarran/django-minipub',
    author='Richard Barran',
    author_email='richard@arbee-design.co.uk',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



import subprocess


def prereleaser_middle(data):
    """
    1. Run the unit tests one last time before we make a release.
    2. Check that code follows PEP8 conventions (more or less).
    """
    print('Running unit tests.')
    subprocess.check_output(["python", "example_project/manage.py", "test", "news", "news_with_archive"])

    print('Running PEP8 check.')
    # See setup.cfg for configuration options.
    subprocess.check_output(["pep8"])
